/*
   fortran.c - FORTRAN language functions for deckmate program.

   Author: Paul Hardy, June-August 2020

   Copyright (C) 2020 Paul Hardy

   LICENSE:

      This program is free software: you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation, either version 2 of the License, or
      (at your option) any later version.

      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.

      You should have received a copy of the GNU General Public License
      along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "config.h"
#include "deckmate.h"


/*
   Draw invariant form markings for FORTRAN listings.
*/
void
new_ps_page_fortran (FILE *outfp) {

   fprintf (outfp, "   %%\n");
   fprintf (outfp, "   %% Template for FORTRAN listings.\n");
   fprintf (outfp, "   %%\n");

   /* Print Statement Number field: 5 columns */
   fprintf (outfp, "   %d %d moveto 0 %d rlineto stroke\n",
           LEFT_MARGIN_80 + 30, TOP_MARGIN_80 + 20, -22);

   fprintf (outfp, "   /Helvetica findfont 5 scalefont setfont\n");

   /* Center on line, not rotated */
   fprintf (outfp, "   (STATEMENT) stringwidth pop\n");
   fprintf (outfp, "   (C/) exch\n");
   fprintf (outfp, "   -2 div %d add %d moveto\n",
            LEFT_MARGIN_80 + 15, TOP_MARGIN_80 + 34);
   fprintf (outfp, "   show\n");
   fprintf (outfp, "   (STATEMENT) dup stringwidth pop\n");
   fprintf (outfp, "   -2 div %d add %d moveto\n",
            LEFT_MARGIN_80 + 15, TOP_MARGIN_80 + 28);
   fprintf (outfp, "   show\n");
   fprintf (outfp, "   (NUMBER) dup stringwidth pop\n");
   fprintf (outfp, "   -2 div %d add %d moveto\n",
            LEFT_MARGIN_80 + 15, TOP_MARGIN_80 + 22);
   fprintf (outfp, "   show\n");

   /* Print Continuation field: 1 column */
   fprintf (outfp, "   %d %d moveto 0 %d rlineto stroke\n",
           LEFT_MARGIN_80 + 36, TOP_MARGIN_80 + 20, -22);

   fprintf (outfp, "   %d %d moveto 90 rotate\n",
           LEFT_MARGIN_80 + 35, TOP_MARGIN_80 + 22);
   fprintf (outfp, "   (CONT.) show\n");
   fprintf (outfp, "   -90 rotate\n");
 
   /* Center on line, not rotated */
   fprintf (outfp, "   (PROGRAM) dup stringwidth pop\n");
   fprintf (outfp, "   -2 div %d add %d moveto\n",
            LEFT_MARGIN_80 + 456, TOP_MARGIN_80 + 28);
   fprintf (outfp, "   show\n");
   fprintf (outfp, "   (IDENTIFICATION) dup stringwidth pop\n");
   fprintf (outfp, "   -2 div %d add %d moveto\n",
            LEFT_MARGIN_80 + 456, TOP_MARGIN_80 + 22);
   fprintf (outfp, "   show\n");

   fprintf (outfp, "   /Helvetica findfont 10 scalefont setfont\n");

   /* Center text on line, adding 3 pt extra for space character (32) */
   fprintf (outfp, "   3 0 32 3 0");

   fprintf (outfp, " (FORTRAN STATEMENT) dup stringwidth pop\n");
   /*
      Adjust starting point to be centered on top of form,
      allowing for 3 pt kerning between 15 letter pairs.
   */
   fprintf (outfp, "   -2 div %f add %d moveto\n",
           LEFT_MARGIN_80 + BOX_WIDTH_80 / 2.0 - 3 * 18 / 2.0 + 0.5,
           TOP_MARGIN_80 + 22);
   fprintf (outfp, "   awidthshow\n");

   /* Print Program Identification field: 8 columns */
   fprintf (outfp, "   %d %d moveto 0 %d rlineto stroke\n",
           LEFT_MARGIN_80 + 432, TOP_MARGIN_80 + 20, -BOX_HEIGHT_80 - 20);

   return;
}


/*
   Print context-dependent vertical rules on a line for FORTRAN.
*/
void
ps_line_out_fortran (FILE *outfp,
                     int line_num, char *inbuffer, language form_lang) {

   int i;  /* loop variable */

   int *verticals;  /* Points to one of the arrays of verticals below */
   int default_verticals[81] = {5, 6, 0};
   int comment_verticals[81] = {0};
   int jcl_verticals[81]     = {0};

   int jcl (char *);


   verticals = NULL;  /* No matching pattern found yet. */

   /*
      Print inter-field vertical lines if not a comment line or JCL.
   */
   if (inbuffer [0] == 'C' || inbuffer [0] == 'c') {
         verticals = comment_verticals;
   }
   else if (jcl (inbuffer) == 1) {  /* If IBM JCL */
         verticals = jcl_verticals;
   }
   else {
         verticals = default_verticals;
   }
   if (verticals != NULL) {
      for (i = 0; verticals[i] != 0; i++) {
         fprintf (outfp, "%d %d tick\n",
                 LEFT_MARGIN_80 + 6 * verticals[i],
                 TOP_MARGIN_80 + 8 - 10 * line_num);
      }
   }

   return;
}

