/*
   input.c - functions to process deckmate input text strings for output.

   Author: Paul Hardy, June-August 2020

   Copyright (C) 2020 Paul Hardy

   LICENSE:

      This program is free software: you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation, either version 2 of the License, or
      (at your option) any later version.

      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.

      You should have received a copy of the GNU General Public License
      along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "config.h"
#include "deckmate.h"


/*
   Determine next line number from carriage control code
   that was in the first column of the current input line.
*/
int
get_next_line (char cc_char, int line_num, int init,
               int fortran_cc, int *cc_tape) {

   int next_line;


   next_line = line_num;
  
   /* Handle FORTRAN carriage control, if option is selected */
   if (fortran_cc == 1) {
      switch (cc_char) {
         case ' ': if (init == 1) {  /* If at top of 1st page */
                      next_line = cc_tape [0];
                   }
                   else {
                      next_line++;
                   }
                   break;   /* Single space      */
         case '0': next_line +=  2; break;   /* Double space      */
         case '-': next_line +=  3; break;   /* Triple space      */
         case '1': /* Advance to next page if not at start of printout */
         case '\f':                         /* <FF> --> new page */
                   if (init == 1) {  /* At the start of the 1st page */
                      next_line  = cc_tape [0];
                   }
                   else {
                      next_line = 66 + cc_tape[0];
                   }
                   break;
         case '\v':next_line +=  6 - (next_line-1) % 6; /* <VT> on 6 lines */
                   break;
         case '+':                 break;   /* No line advance   */

                   /* Handle carriage control tape positions 2-12 */
         case '2':
         case '3':
         case '4':
         case '5':
         case '6':
         case '7':
         case '8':
         case '9':
         case 'A':
         case 'B':
         case 'C':
         case 'a':
         case 'b':
         case 'c':
            if (cc_char <= '9') {
               next_line = cc_tape [cc_char - '0' - 1];
            }
            else {  /* A -> tab stop 9, B -> tab stop 10, C -> tab stop 11 */
               next_line = cc_tape [(cc_char | 0x60) - 'a' + 9];
            }
            if (next_line < line_num) {  /* skip to that line on next page */
               next_line += 66;
            }
            break;

         default:  next_line++;    break;   /* Single space      */
      }
   }
  

   return next_line;
}


/*
   Take an input string that can create backspaces and convert it
   into two overprinted strings plus one boldface string.
*/
void
insplit (char *inbuffer, char *line1, char *line2, unsigned fonts_used) {

   int i;   /* input  string index */
   int j;   /* output string index */


   line1[0] = inbuffer[0];
   line2[0] = ' ';

   for (i = j = 1;
        inbuffer[i] != '\n' && inbuffer[i] != '\r' && inbuffer[i] != '\0';
        i++) {

      /*
         If unrecognized character in ISO-IR-68 APL file,
         replace with squish quad character combination.
      */
      if (((fonts_used & FONT_PS_DOTWRITERAPL) != 0) &&
          (unsigned)inbuffer[i] >= 0x7F) {
         line1[j] = '\073';
         line2[j] = '\047';
      }
      else {
         line1[j] = inbuffer[i];
         if (inbuffer[i+1] == '\b') {
            line2[j] = inbuffer[i+2];
            i += 2;
         }
         else {
            line2[j] = ' ';
         }
      }
      j++;
   }
   line1[j] = line2[j] = '\0';

   return;
}

