/*
   params.c - parse command line parameters to the deckmate program.

   Author: Paul Hardy, June-August 2020

   Copyright (C) 2020 Paul Hardy

   LICENSE:

      This program is free software: you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation, either version 2 of the License, or
      (at your option) any later version.

      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.

      You should have received a copy of the GNU General Public License
      along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#include "config.h"
#include "deckmate.h"


/*
   Parse command line parameters.

   Return a count of the number of errors encountered; 0 = no errors.
*/
int parse_params (int argc, char **argv,
                  char *infile, char *outfile, FILE **infp, FILE **outfp,
                  language *form_lang, char *ddname,
                  int *border, int *colnums, int *pagenums,
                  struct ps_rgb *rgb, int *tab_stops, int *ntabs,
                  char *listing_title, int *wide,
                  int *fortran_cc, int *cc_tape, unsigned *fonts_used,
                  char *listing_font, char *special_font) {


   int i, j;  /* loop variables */

   /*
      Variables settable by program arguments.
   */
   char *lang_arg;                      /* language given as argument       */
   char *rgb_arg;                       /* R,G,B color given as argument    */
   double red_arg, green_arg, blue_arg; /* separated colors from rgb_arg    */
   int  tempnum;       /* temporary integer for carriage control parameters */
   int  cc_index;      /* index into cc_tape parameter array                */

   int error_count;  /* return value from parsing parameters; 0 = no errors */

   error_count = 0;

   *ntabs = 0;

   *infp  = stdin;
   *outfp = stdout;

   strcpy (listing_font, "Courier");
   strcpy (special_font, "Symbol");

   /*
      Parse program arguments.
   */
   for (i = 1; i < argc; i++) {
      if (argv[i][0] == '-') {

         switch (argv[i][1]) {

            case 'i':  /* input filename specified */
               infile = argv[++i];
               if (i == argc || argv[i][0] == '-') {
                  fprintf (stderr, "ERROR: \"-i\": next argument must be ");
                  fprintf (stderr, "the input filename.\n\n");
                  fprintf (stderr,
                           "       Invoke %s with \"-h\" for options.\n\n",
                           argv[0]);
                  exit (EXIT_FAILURE);
               }
               if ((*infp = fopen (infile, "r")) == NULL) {
                  fprintf (stderr,
                           "ERROR: Input file \"%s\" does not exist.\n\n",
                           infile);
                  if (*outfp != NULL) fclose (*outfp);
                  exit (EXIT_FAILURE);
               }
               /*
                  Exclude path component from filename for PostScript
                  header output.
               */
               for (j = strlen (infile) - 1;
                    j > 0 && infile[j] != '/';
                    j--);

               if (j < 0) j = 0;
               if (infile[j] == '/') j++;
               strncpy (listing_title, &infile[j], MAXTITLE);
               listing_title [MAXTITLE - 1] = '\0';
               break;

            case 'o': /* output filename specified */
               outfile = argv[++i];
               if (i == argc || argv[i][0] == '-') {
                  fprintf (stderr, "ERROR: \"-o\": next argument must be ");
                  fprintf (stderr, "the output filename.\n\n");
                  fprintf (stderr,
                           "       Invoke %s with \"-h\" for options.\n\n",
                           argv[0]);
                  exit (EXIT_FAILURE);
               }
               if (outfile[0] != '\0') {  /* open input file */
                  if ((*outfp = fopen (outfile, "w")) == NULL) {
                     fprintf (stderr,
                              "ERROR: Can't open %s for output.\n\n",
                              outfile);
                     if (*infp != NULL) fclose (*infp);
                     exit (EXIT_FAILURE);
                  }
               }
               break;

            case 'F':  /* font for listings */
               if (strcmp (argv[i+1], "Cardpunch") == 0) {
                  strcpy (listing_font, "Cardpunch");
                  strcpy (special_font, "Cardpunch");
                  *fonts_used = *fonts_used | FONT_PS_CARDPUNCH;
               }
               else if (strcmp (argv[i+1], "DOTwriterAPL") == 0) {
                  strcpy (listing_font, "DOTwriterAPL");
                  strcpy (special_font, "DOTwriterAPL");
                  *fonts_used = *fonts_used | FONT_PS_DOTWRITERAPL;
               }
               else if (strcmp (argv[i+1], "DOTwriter") == 0) {
                  strcpy (listing_font, "DOTwriter");
                  strcpy (special_font, "DOTwriter");
                  *fonts_used = *fonts_used | FONT_PS_DOTWRITER;
               }
               else {
                  fprintf (stderr, "Unrecognized font %s for \"-F\" option.\n",
                           argv[i+1]);
                  fprintf (stderr, "Using default fonts.\n\n");
               }

               i++;
               break;

            case 'f':  /* form overrides */
               if (strncmp (argv[i+1], "nob", 3) == 0) {
                  *border = 0;  /* no border around page contents */
               }
               else if (strncmp (argv[i+1], "noc", 3) == 0) {
                  *colnums = 0;  /* no column numbers at top of page */
               }
               else if (strncmp (argv[i+1], "nop", 3) == 0) {
                  *pagenums = 0;  /* no page numbers at bottom of page */
               }

               i++;
               break;

            case 'l':  /* language for template specified */
               lang_arg = argv[++i];
               if (i == argc || argv[i][0] == '-') {
                  fprintf (stderr, "ERROR: \"-l\": next argument must be ");
                  fprintf (stderr, "the language template name.\n\n");
                  fprintf (stderr,
                           "       Invoke %s with \"-h\" for options.\n\n",
                           argv[0]);
                  exit (EXIT_FAILURE);
               }
               switch (lang_arg[0]) {
                  case 'f': *form_lang = LANG_FORTRAN; break;
                  case 'c': *form_lang = LANG_COBOL;   break;
                  case 'r': *form_lang = LANG_RPG;     break;
                  default:  break;
               }
               break;

            /*
               Specify RGB PostScript color for stripes.
               Use -c0,0,0 for white (i.e., no colored stripes).
            */
            case 'c':  /* RGB color triplet specified */
               rgb_arg = argv[++i];
               if (i == argc || argv[i][0] == '-') {
                  fprintf (stderr, "ERROR: \"-c\": next argument must be ");
                  fprintf (stderr, "the Red,Green,Blue color triplet.\n\n");
                  fprintf (stderr,
                           "       Invoke %s with \"-h\" for options.\n\n",
                           argv[0]);
                  exit (EXIT_FAILURE);
               }
               sscanf (rgb_arg,"%lf,%lf,%lf", &red_arg, &green_arg, &blue_arg);
               if (red_arg   < 0.0)  red_arg   = 0.0;
               if (red_arg   > 1.0)  red_arg   = 1.0;
               if (green_arg < 0.0)  green_arg = 0.0;
               if (green_arg > 1.0)  green_arg = 1.0;
               if (blue_arg  < 0.0)  blue_arg  = 0.0;
               if (blue_arg  > 1.0)  blue_arg  = 1.0;
               rgb->red   = red_arg;
               rgb->green = green_arg;
               rgb->blue  = blue_arg;
               break;

            /* Specify line printer carriage control positions */
            /*
               If FORTRAN (ASA/ANSI) carriage control, first column
               of each input line is a FORTRAN carriage control code.
            */
            case 'C':
               *fortran_cc = 1;
               /*
                  If next argument starts with an integer, process as
                  a list of line numbers for carriage control.
               */
               if ((i + 1) < argc && isdigit (argv[i+1][0])) {
                  i++;
                  j = 0;
                  cc_index = 0;
                  do {
                     tempnum = atoi (&argv[i][j]);
                     if (tempnum >= 1 && tempnum <= 66) {
                        cc_tape [cc_index] = tempnum;
                     }
                     else {
                        fprintf (stderr,
                           "ERROR: invalid carriage control line number at %s\n",
                           &argv[i][j]);
                        exit (EXIT_FAILURE);
                     }
                     cc_index++;
                     while (argv[i][j] != '\0' && argv[i][j] != ',') j++;
                     if (argv[i][j] == ',') j++;
                  }  while (argv[i][j] != '\0');
               }  /* if (i < (argc - 1) && isdigit (argv[i+1][0])) */

               break;

            /* print help menu and exit */
            case 'h':
               printf ("\n   %s arguments:\n\n", PACKAGE_STRING);
               printf ("      -i input_file      Specify input filename.\n");
               printf ("      -o output_file     Specify output filename.\n");
               putchar ('\n');
               printf ("      -c R,G,B           Specify Red,Green,Blue intensities of three-line\n");
               printf ("                         colored stripes in range 0.0 (completely off) to\n");
               printf ("                         1.0, inclusive.  The default is 0.75,1.0,0.75.\n");
               putchar ('\n');
               printf ("      -F name            Use alternate deckmate font \"name\" for listings.\n");
               printf ("                         Custom fonts available are \"Cardpunch\", \"DOTwriter\",\n");
               printf ("                         and \"DOTwriterAPL\".  The default is the PostScript\n");
               printf ("                         Courier font.\n");
               putchar ('\n');
               printf ("      -f nob[order]      Do not print border around listing contents.\n");
               printf ("      -f noc[olumns]     Do not print column numbers at top of page.\n");
               printf ("      -f nop[agenumbers] Do not print page numbers at bottom of page.\n");
               putchar ('\n');
               printf ("      -h                 Print this help message and exit.  See the man page\n");
               printf ("                         for more information.\n");
               putchar ('\n');
               printf ("      -l c[obol]         Generate output using COBOL template.\n");
               printf ("      -l f[ortran]       Generate output using FORTRAN template.\n");
               printf ("      -l r[pg]           Generate output using RGB template.\n");
               putchar ('\n');
               printf ("      -C[C]              Each line begins with an ASA/ANSI (\"FORTRAN\")\n");
               printf ("                         carriage control character.\n");
               putchar ('\n');
               printf ("      -t n1,n2,...       Specify tab stop column numbers.  A dashed gray\n");
               printf ("                         line will be printed down the length of the form\n");
               printf ("                         at each specified column position.\n");
               putchar ('\n');
               printf ("      -v, --version      Print version information.  Exit if this was\n");
               printf ("                         the only program argument.\n");
               putchar ('\n');
               printf ("      -w                 Wide mode, 132 columns.  Default is 80 columns.\n");
               putchar ('\n');
               printf ("      DD=[stepname.]ddname  Search for a JCL line beginning with \"//ddname\"\n");
               printf ("                         or \"//stepname.ddname\" as marking the start of data\n");
               printf ("                         set input for a program.  Once found, stop printing\n");
               printf ("                         language-specific solid vertical field marks.\n");
               putchar ('\n');
               exit (EXIT_SUCCESS);
               break;

            case 't':  /* store tab stops to print */
               i++;
               if (i == argc || argv[i][0] == '-') {
                  fprintf (stderr, "ERROR: \"-t\": next argument must be ");
                  fprintf (stderr, "a list of columns for dashed lines.\n");
                  fprintf (stderr,
                           "       Invoke %s with \"-h\" for options.\n\n",
                           argv[0]);
                  exit (EXIT_FAILURE);
               }
               for (j = 0; argv[i][j] >= '0' && argv[i][j] <= '9'; j++) {
                  tab_stops[*ntabs] = atoi (&argv[i][j]);
                  if (tab_stops[*ntabs] < 1 || tab_stops[*ntabs] > 133) {
                     fprintf (stderr,
                              "ERROR: tab stop out of range 1..133 at %s\n",
                              &argv[i][j]);
                     if (*infp  != NULL) fclose (*infp);
                     if (*outfp != NULL) fclose (*outfp);
                     exit (EXIT_FAILURE);
                  }
                  (*ntabs)++;
                  while (isdigit (argv[i][j])) j++; /* Skip over this number */
               }
               tab_stops[*ntabs] = 0;  /* mark end of list */
               break;

            /* print program version string */
            case 'v':
               printf ("%s - Visit %s for information.\n",
                       PACKAGE_STRING, PACKAGE_URL);
               break;

            /* wide mode: 132 column output */
            case 'w':
               *wide = 1;
               break;

            case '-':  /* argument starts with "--" */
               if (strncmp (argv[i], "--version", 10) == 0) {
                  printf ("%s - Visit %s for information.\n",
                          PACKAGE_STRING, PACKAGE_URL);
                  /* Exit if this was the only argument. */
                  if (argc == 2) {
                     exit (EXIT_SUCCESS);
                  }
               }
               break;

            default: /* print help message and exit */
               fprintf (stderr, "Unknown argument %s\n", argv[i]);
               fprintf (stderr, "Type \"%s -h\" for help menu.\n", argv[0]);
               if (*infp  != NULL) fclose (*infp);
               if (*outfp != NULL) fclose (*outfp);
               exit (EXIT_FAILURE);
               break;

         }  /* switch argv[i][1] */

      }  /* if argument starts with '-' */
      /*
         See if a JCL DD field is specified as beginning data
         input.  From that point on language-specific solid
         vertical lines will not be printed.  The DD statement
         <ddname> field can be preceded by an optional
         <stepname> + '.'.  This gives a maximum string size
         of <stepname> + '.' + <ddname> + '\0' =
                8      +  1  +     8    +  1   = 18 bytes.
      */
      else if (strncmp (argv[i], "DD=", 3) == 0) {
         (void)strncpy (ddname, &argv[i][3], 18);
         ddname[17] = '\0';
      }

   }  /* done scanning program arguments */

   if (*infp  == NULL)  *infp  = stdin;
   if (*outfp == NULL)  *outfp = stdout;

   if (*infp == stdin) {
      strcpy (listing_title, "stdin");
   }


   return 0;
}

