/*
   printer.c - PostScript form printout functions for deckmate.

   Author: Paul Hardy, June-August 2020

   Copyright (C) 2020 Paul Hardy

   LICENSE:

      This program is free software: you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation, either version 2 of the License, or
      (at your option) any later version.

      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.

      You should have received a copy of the GNU General Public License
      along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "config.h"
#include "deckmate.h"


/*
   Start a new PostScript file.

   Inputs:

      outfp          File pointer for output file
      listing_title  %%Title string for document header
      creator        %%Creator string for document header
      url            URL for %%Creator string
      bbx            Bounding box width
      bby            Bounding box height
*/
void
new_ps_file (FILE *outfp, char *listing_title, char *creator, char *url,
             int bbx, int bby, char *dsc_version) {

   fprintf (outfp, "%%!PS-Adobe-%s\n", dsc_version);
   fprintf (outfp, "%%%%DocumentFonts: (atend)\n");
   fprintf (outfp, "%%%%Title: PostScript conversion of %s\n", listing_title);
   fprintf (outfp, "%%%%Creator: %s (%s)\n", creator, url);
   fprintf (outfp, "%%%%Pages: (atend)\n");
   fprintf (outfp, "%%BoundingBox: 0 0 %d %d\n", bbx, bby);
   fprintf (outfp, "%%%%EndComments\n");

   return;
}


/*
   new_ps_prolog - output the template to print on each page,
                   in the prolog section of a PostScript file.
*/
void
new_ps_prolog (FILE *outfp,
               language form_lang, int border, int colnums, int pagenums,
               struct ps_rgb *rgb, int *tab_stops,
               int page_num, int wide,
               char *listing_font, char *special_font) {

   int i;  /* loop variable */

   char *col1_80="00000000011111111112222222222333333333344444444445555555555666666666677777777778";
   char *col2_80="12345678901234567890123456789012345678901234567890123456789012345678901234567890";
   char *col1_132="                                                                                                   111111111111111111111111111111111";
   char *col2_132="000000000111111111122222222223333333333444444444455555555556666666666777777777788888888889999999999000000000011111111112222222222333";
   char *col3_132="123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012";

   int left_margin, top_margin, box_width, box_height;

   void font_cardpunch      (FILE *outfp);
   void font_dotwriter      (FILE *outfp);
   void new_ps_page_tabs    (FILE *outfp, int *tab_stops, int wide);
   void new_ps_page_fortran (FILE *outfp);
   void new_ps_page_cobol   (FILE *outfp);
   void new_ps_page_rpg     (FILE *outfp);


   if (wide == 0) {  /* 80 column output */
      left_margin = LEFT_MARGIN_80;
       top_margin =  TOP_MARGIN_80;
       box_width  =   BOX_WIDTH_80;
       box_height =  BOX_HEIGHT_80;
   }
   else {  /* 132 column output */
      left_margin = LEFT_MARGIN_132;
       top_margin =  TOP_MARGIN_132;
       box_width  =   BOX_WIDTH_132;
       box_height =  BOX_HEIGHT_132;
   }

   fprintf (outfp, "%%%%BeginProlog\n");

   /* Default line style for the entire document */
   fprintf (outfp, "%% linestyle -- set document line defaults\n");
   fprintf (outfp,
            "/linestyle { 0.5 setlinewidth 2 setlinecap } bind def\n");

   /*
      Alternate between 3 lines of green background and
      3 lines of white background, after 6 white lines
      at the top.
   */
   if (rgb->red == 0.0 && rgb->green == 0.0 && rgb->blue == 0.0) {
      fprintf (outfp, "%% stripes -- empty function for a white page\n");
      fprintf (outfp, "/stripes { } bind def\n");
   }
   else {  /* non-white color */
      fprintf (outfp, "%% <width> stripe -- draw a 30 pt tall color stripe from upper left-hand corner\n");
      fprintf (outfp, "/stripe {\n");  /* call with width of box on stack */
      fprintf (outfp,
               "   dup 0 rlineto 0 -30 rlineto neg 0 rlineto closepath fill\n");
      fprintf (outfp, "} bind def\n");
      fprintf (outfp, "%% stripes -- draw color stripe bars on new page\n");
      fprintf (outfp, "/stripes {\n");
      fprintf (outfp, "   gsave\n");
      fprintf (outfp, "   %% Draw ten 3-line color stripes on new page from top to bottom\n");
      /* Color stripe */
      fprintf (outfp, "   %.4f %.4f %.4f setrgbcolor\n",
               rgb->red, rgb->green, rgb->blue);
      for (i = 7; i < 66; i += 6) {  /* Alternate green and white paper */
         fprintf (outfp, "   %d %d moveto %d stripe\n",
                 left_margin, top_margin + 8 - 10 * i, box_width);
      }
      fprintf (outfp, "   grestore  %% Restore previous color\n");
      fprintf (outfp, "} bind def\n");
   }

   /*
      Dashed lines for tabs specified with the "-t"
      command line argument.
   */
   fprintf (outfp,
            "%% usertabs -- draw dashed gray lines at specified columns\n");
   fprintf (outfp, "/usertabs {\n");

   /* Print user-defined tab stop columns of dashed lines on form */
   if (tab_stops [0] > 0) {
      new_ps_page_tabs (outfp, tab_stops, wide);
   }

   fprintf (outfp, "} bind def\n");

   fprintf (outfp, "%% pageborders -- draw page borders & column legend\n");
   fprintf (outfp, "/pageborders { \n");
   /*
      Draw invariant lines and text for all forms.
   */
   if (border == 1) {
      /* Draw outer box around listing */
      fprintf (outfp, "   %% Draw outer border around listing.\n");
      fprintf (outfp, "   %d %d moveto\n", left_margin, top_margin - 2);
      fprintf (outfp, "      %d 0 rlineto",  box_width );
      fprintf (outfp, " 0 %d rlineto", -box_height + 2);
      fprintf (outfp, " %d 0 rlineto", -box_width     );
      fprintf (outfp, " 0 %d rlineto",  box_height - 2);
      fprintf (outfp, " closepath\n");
      fprintf (outfp, "   stroke\n");
   }  /* border == 1 */

   /*
      Complete rectangle over column numbers:

         80-column printouts have 2 rows of column numbers,  01..80.
         132-column printouts have 3 rows of column numbers, 01..132.
   */
   if (colnums == 1) {
      fprintf (outfp, "   %% Draw border on top of column numbers.\n");
      fprintf (outfp, "   %d %d moveto\n", left_margin, top_margin - 2);

       if (wide == 0)
          fprintf (outfp,
                   "      0 22 rlineto %d 0 rlineto 0 -22 rlineto closepath\n", box_width);
       else
          fprintf (outfp,
                   "      0 32 rlineto %d 0 rlineto 0 -32 rlineto closepath\n", box_width);

      fprintf (outfp, "   stroke\n");

      fprintf (outfp, "   %% Draw column numbers.\n");
      /* Draw column numbers */
      if (wide == 0) {
         fprintf (outfp, "   /Courier findfont 10 scalefont setfont\n");
         fprintf (outfp, "   %d %d moveto\n", left_margin, top_margin + 10);
         fprintf (outfp, "   (%s) show\n", col1_80);
         fprintf (outfp, "   %d %d moveto\n", left_margin, top_margin);
         fprintf (outfp, "   (%s) show\n", col2_80);
      }
      else {
         fprintf (outfp, "   /Courier findfont 10 scalefont setfont\n");
         fprintf (outfp, "   %d %d moveto\n", left_margin, top_margin + 20);
         fprintf (outfp, "   (%s) show\n", col1_132);
         fprintf (outfp, "   %d %d moveto\n", left_margin, top_margin + 10);
         fprintf (outfp, "   (%s) show\n", col2_132);
         fprintf (outfp, "   %d %d moveto\n", left_margin, top_margin);
         fprintf (outfp, "   (%s) show\n", col3_132);
      }
   }  /* colnums == 1 */

   fprintf (outfp, "} bind def\n");

   /*
      Language-specific labels and markings for an 80-column page.
   */

   /*
      Draw vertical stroke one line high within an output line
      in a language-specific file  to mark a field.
   */
   if (form_lang != LANG_NONE) {
      /* Call tick with x y position on top of stack */
      fprintf (outfp, "%% <x> <y> tick - draw one-line vertical stroke downward\n");
      fprintf (outfp, "/tick { moveto 0 -10 rlineto stroke } bind def\n");
   }

   /*
      Define form template specific to the programming language.
   */
   fprintf (outfp,
        "%% langtemplate -- form template for supported programming languages\n");
   fprintf (outfp, "/langtemplate {\n");

   if (wide == 0) {
      switch (form_lang) {
         case LANG_NONE:
                 break;
         case LANG_FORTRAN:
                 new_ps_page_fortran (outfp);
                 break;
         case LANG_COBOL:
                 new_ps_page_cobol (outfp);
                 break;
         case LANG_RPG:
                 new_ps_page_rpg (outfp);
                 break;
         default:
                 break;
      }
   }

   fprintf (outfp, "} bind def\n");

   /*
      Use PostScript's Courier built-in font for listings,
      or a font included with this package.
   */
   if (strncmp (listing_font, "DOTwriterAPL", 12) == 0) {
      fprintf (outfp, "%% listing -- define font for rendering program listings\n");
      fprintf (outfp,
               "/listing { /DOTwriterAPL findfont 10 scalefont setfont } def\n");
      fprintf (outfp,
               "/bold { /DOTwriterAPL-Bold findfont 10 scalefont setfont } bind def\n");
      fprintf (outfp, "%% special -- define font for rendering Symbol glyphs\n");
      fprintf (outfp,
               "/special { /DOTwriterAPL findfont 10 scalefont setfont } def\n");
   }
   else if (strncmp (listing_font, "DOTwriter", 9) == 0) {
      fprintf (outfp, "%% listing -- define font for rendering program listings\n");
      fprintf (outfp,
               "/listing { /DOTwriter findfont 10 scalefont setfont } def\n");
      fprintf (outfp,
               "/bold { /DOTwriter-Bold findfont 10 scalefont setfont } bind def\n");
      fprintf (outfp, "%% special -- define font for rendering Symbol glyphs\n");
      fprintf (outfp,
               "/special { /DOTwriter findfont 10 scalefont setfont } def\n");
   }
   else if (strncmp (listing_font, "Cardpunch", 9) == 0) {
      fprintf (outfp, "%% listing -- define font for rendering program listings\n");
      fprintf (outfp,
               "/listing { /Cardpunch findfont 10 scalefont setfont } def\n");
      fprintf (outfp,
               "/bold { /Cardpunch-Bold findfont 10 scalefont setfont } bind def\n");
      fprintf (outfp, "%% special -- define font for rendering Symbol glyphs\n");
      fprintf (outfp,
               "/special { /Cardpunch findfont 10 scalefont setfont } def\n");
   }
   else {
      fprintf (outfp, "%% listing -- define font for rendering program listings\n");
      fprintf (outfp,
               "/listing { /Courier findfont 10 scalefont setfont } bind def\n");
      fprintf (outfp,
               "/bold { /Courier-Bold findfont 10 scalefont setfont } bind def\n");
      fprintf (outfp, "%% special -- define font for rendering Symbol glyphs\n");
      fprintf (outfp,
               "/special { /Symbol findfont 10 scalefont setfont } bind def\n");
   }


   fprintf (outfp, "%% <string> bsp -- backspace to start of <string>\n");
   fprintf (outfp,
            "/bsp { stringwidth pop neg 0 rmoveto } bind def\n");

   fprintf (outfp, "%% <(Page x)> footer -- draw footer string 1/2 inch up from page bottom\n");
   fprintf (outfp, "/footer {\n");
   if (pagenums == 1) {
      fprintf (outfp, "   gsave\n");
      fprintf (outfp, "   /Courier findfont 12 scalefont setfont\n");
      fprintf (outfp, "   dup stringwidth pop\n");
      /* Center page #: 80/2  col + 66 pt or 132/2 column widths + 108 pt */
      fprintf (outfp, "   -2 div %d add 36 moveto show\n",
               wide == 0 ? PAGE_MIDDLE_80 : PAGE_MIDDLE_132);
      fprintf (outfp, "   grestore\n");
   }
   fprintf (outfp, "} bind def\n");

   fprintf (outfp, "%% pageframe -- complete template to execute at start of each new page\n");
   fprintf (outfp, "/pageframe {\n");
   fprintf (outfp, "   linestyle stripes usertabs pageborders langtemplate listing\n");
   fprintf (outfp, "} def\n");

   fprintf (outfp, "%%%%EndProlog\n");

   return;
}


/*
   Setup area in PostScript file, after Prolog section.
   Include optional fonts here.
*/
void
new_ps_setup (FILE *outfp, char *listing_font, char *special_font) {

   void font_cardpunch    (FILE *outfp);
   void font_dotwriter    (FILE *outfp);
   void font_dotwriterapl (FILE *outfp);


   if (strncmp (listing_font, "Cardpunch", 9) == 0) {
      fprintf (outfp, "%%%%BeginSetup\n");
      font_cardpunch (outfp);
      fprintf (outfp, "%%%%EndSetup\n");
   }
   else if (strncmp (listing_font, "DOTwriterAPL", 12) == 0) {
      fprintf (outfp, "%%%%BeginSetup\n");
      font_dotwriterapl (outfp);
      fprintf (outfp, "%%%%EndSetup\n");
   }
   else if (strncmp (listing_font, "DOTwriter", 9) == 0) {
      fprintf (outfp, "%%%%BeginSetup\n");
      font_dotwriter (outfp);
      fprintf (outfp, "%%%%EndSetup\n");
   }

   return;
}


/*
   Start a new PostScript page.
*/
void
new_ps_page (FILE *outfp,
             language form_lang, int border, int colnums,
             struct ps_rgb *rgb, int *tab_stops,
             int page_num, int wide) {

   fprintf (outfp, "%%%%Page: %d %d\n", page_num, page_num);

   if (wide == 1) {
      fprintf (outfp, "0 800 translate -90 rotate %f %f scale\n",
               WIDE_SCALE, WIDE_SCALE);
   }

   fprintf (outfp, "pageframe\n");

   return;
}


/*
   End the current PostScript page.
*/
void
end_ps_page (FILE *outfp, int page_num, int wide) {

   /*
      Print page number centered across bottom of page.
      8.5 inches = 2 * 306 pt; this will be off by
      about 3 mm for A4 paper.
   */
   fprintf (outfp, "(Page %d) footer\n", page_num);

   fprintf (outfp, "showpage\n");

   return;
}


/*
   End the current PostScript output file.
*/
void
end_ps_file (FILE *outfp, int npages, unsigned fonts_used) {


   fprintf (outfp, "%%%%Trailer\n");
   fprintf (outfp, "%%%%Pages: %d\n", npages);

   fprintf (outfp, "%%%%DocumentFonts:");
   if ((fonts_used & FONT_PS_COURIER)           != 0)
      fprintf (outfp, " Courier");
   if ((fonts_used & FONT_PS_COURIER_BOLD)      != 0)
      fprintf (outfp, " Courier-Bold");
   if ((fonts_used & FONT_PS_HELVETICA)         != 0)
      fprintf (outfp, " Helvetica");
   if ((fonts_used & FONT_PS_HELVETICA_BOLD)    != 0)
      fprintf (outfp, " Helvetica-Bold");
   if ((fonts_used & FONT_PS_SYMBOL)            != 0)
      fprintf (outfp, " Symbol");
   if ((fonts_used & FONT_PS_SYMBOL_BOLD)       != 0)
      fprintf (outfp, " Symbol-Bold");
   if ((fonts_used & FONT_PS_CARDPUNCH)         != 0)
      fprintf (outfp, " Cardpunch");
   if ((fonts_used & FONT_PS_CARDPUNCH_BOLD)    != 0)
      fprintf (outfp, " Cardpunch-Bold");
   if ((fonts_used & FONT_PS_DOTWRITER)         != 0)
      fprintf (outfp, " DOTwriter");
   if ((fonts_used & FONT_PS_DOTWRITER_BOLD)    != 0)
      fprintf (outfp, " DOTwriter-Bold");
   if ((fonts_used & FONT_PS_DOTWRITERAPL)      != 0)
      fprintf (outfp, " DOTwriterAPL");
   if ((fonts_used & FONT_PS_DOTWRITERAPL_BOLD) != 0)
      fprintf (outfp, " DOTwriterAPL-Bold");
   fputc ('\n', outfp);

   if ((fonts_used & FONT_PS_CARDPUNCH) != 0) {
      fprintf (outfp, "%%%%DocumentSuppliedFonts: Cardpunch Cardpunch-Bold\n");
   }
   if ((fonts_used & FONT_PS_DOTWRITER) != 0) {
      fprintf (outfp, "%%%%DocumentSuppliedFonts: DOTwriter DOTwriter-Bold\n");
   }
   if ((fonts_used & FONT_PS_DOTWRITERAPL) != 0) {
      fprintf (outfp, "%%%%DocumentSuppliedFonts: DOTwriterAPL DOTwriterAPL-Bold\n");
   }

   fprintf (outfp, "%%%%EOF\n");

   return;
}


/*
   Print an input line consisting of one string or a combination
   of two strings.  If any character is the same in both input
   strings, it will be printed as boldface.
*/
void
ps_line_out (FILE *outfp, int line_num, int fortran_cc,
             char *chbuffer1, char *chbuffer2, language form_lang,
             int wide, int *symbol, unsigned *fonts_used) {

   int i;          /* loop variable */
   int chars_per_line;
   char instring [3][2 * MAXBUFFER];  /* working buffers for input strings  */
   char outstring[3][2 * MAXBUFFER];  /* working buffers for output strings */

   void ps_escape (char *inbuffer, int chars_per_line, int fortran_cc,
                   char *safe_buffer, int *symbol, unsigned *fonts_used);

   void ps_string_out (FILE *outfp, char *outstring, int symbol, int bold);

   /* Language-specific functions */
   void ps_line_out_fortran (FILE *, int, char *, language);
   void ps_line_out_cobol   (FILE *, int, char *, language);
   void ps_line_out_rpg     (FILE *, int, char *, language);


   if (wide)
      chars_per_line = 132;
   else
      chars_per_line =  80;

   /*
      Copy input strings to 2 non-boldface output strings, plus
      one boldface string wherever the 2 input strings match.
   */
   for (i = 0; chbuffer1[i] != '\0' && chbuffer2[i] != '\0'; i++) {
      if (chbuffer1[i] == chbuffer2[i]) {  /* printing same character twice */
         instring[2][i] = chbuffer1[i];    /* print as boldface             */
         instring[0][i] = instring[1][i] = ' ';
      }
      else {
         instring[2][i] = ' ';             /* characters differ; not bold */
         instring[0][i] = chbuffer1[i];
         instring[1][i] = chbuffer2[i];
      }
   }
   instring[2][i] = '\0';  /* at least one input string ended; boldface done */

   /*
      One or both input strings have ended.
   */
   if (chbuffer1[i] == '\0') {
      instring[0][i] = '\0';
      while (chbuffer2[i] != '\0') {  /* copy rest of second string */
         instring[1][i] = chbuffer2[i];
         i++;
      }
      instring[1][i] = '\0';
   }

   if (chbuffer2[i] == '\0') {
      instring[1][i] = '\0';
      while (chbuffer1[i] != '\0') {  /* copy rest of first string */
         instring[0][i] = chbuffer1[i];
         i++;
      }
      instring[0][i] = '\0';
   }

   /*
      Remove trailing spaces from input lines for more efficient printing.
   */
   for (i = strlen (instring[0]) - 1; i >= 0 && (unsigned)instring[0][i] <= ' '; i--) {
      instring[0][i] = '\0';
   }
   for (i = strlen (instring[1]) - 1; i >= 0 && (unsigned)instring[1][i] <= ' '; i--) {
      instring[1][i] = '\0';
   }
   for (i = strlen (instring[2]) - 1; i >= 0 && (unsigned)instring[2][i] <= ' '; i--) {
      instring[2][i] = '\0';
   }

   /*
      If there is a boldface character, note that we needed
      the boldface version of the listing font for the
      "%%DocumentFonts" line in the trailer portion of the
      PostScript file.
   */
   if (instring[2][0] != '\0') {
      /*
         If the listing font is not Courier, then that alternate
         listing font is the one that is boldface.
      */
      if ((*fonts_used & FONT_PS_CARDPUNCH)           != 0)
         *fonts_used |= FONT_PS_CARDPUNCH_BOLD;

      else if ((*fonts_used & FONT_PS_DOTWRITER)      != 0)
            *fonts_used |= FONT_PS_DOTWRITER_BOLD;

      else if ((*fonts_used & FONT_PS_DOTWRITERAPL)   != 0)
            *fonts_used |= FONT_PS_DOTWRITERAPL_BOLD;
      /*
         Note: the Symbol-Bold font is never used.  The logical not
         character is printed in non-boldface but offset slightly
         vertically.  This is because the Symbol-Bold version of
         that character is wider than the space in the Courier font.
         The only remaining font will be Courier-Bold but test that
         Courier is set as a font used for proper operation verification.
      */
      else if ((*fonts_used & FONT_PS_COURIER)        != 0)
         *fonts_used |= FONT_PS_COURIER_BOLD;
   }

   for (i = 0; i < 3; i++) {
      if (instring[i][0] == '\0') {
         outstring[i][0] = '\0';
      }
      else {
         ps_escape (instring [i], chars_per_line, fortran_cc,
                    outstring[i], symbol, fonts_used);
      }
   }

   /*
      Print outstring[0], outstring[1], and outstring[2].
      outstring[2] is the boldface string for double-printed glyphs.
   */
   for (i = 0; i < 3; i++) {

      if (outstring[i][0] != '\0') {
         /*
            Move to start of printing position on line.
         */
         if (wide == 0) {
            fprintf (outfp, "%d %d moveto\n",
                    LEFT_MARGIN_80,
                    TOP_MARGIN_80 - 10 * line_num);
         }
         else {
            fprintf (outfp, "%d %d moveto\n",
                    LEFT_MARGIN_132,
                    TOP_MARGIN_132 - 10 * line_num);
         }

         if (i == 2) fprintf (outfp, "gsave bold\n");
         ps_string_out (outfp, outstring[i], *symbol, i == 2);
         if (i == 2) fprintf (outfp, "grestore\n");
      }
   }

   /*
      Language-specific labels and markings for a page.
      Use the text from the first original input string.
   */
   switch (form_lang) {
      case LANG_NONE: break;
      case LANG_FORTRAN:
              ps_line_out_fortran (outfp, line_num, chbuffer1, form_lang);
              break;
      case LANG_COBOL:
              ps_line_out_cobol   (outfp, line_num, chbuffer1, form_lang);
              break;
      case LANG_RPG:
              ps_line_out_rpg     (outfp, line_num, chbuffer1, form_lang);
              break;
      default: break;
   }


   return;
}


/*
   Print one string out from current location.
*/
void
ps_string_out (FILE *outfp, char *outstring, int symbol, int bold) {

   char *string_start;
   char *symbol_loc;
   char *string_end;


   /*
      Print input line.
   */
   if (symbol == 1) {
      string_start = outstring;
      while (*string_start != '\0') {

         /* find longest substring up to Symbol font glyph */
         string_end = strstr (string_start, "\\330");

         if (string_end == NULL) { /* Print remainder of string */
            if (*string_start != '\0')
               fprintf (outfp, "(%s) show\n", string_start);
            string_start = &outstring [strlen (outstring)];/*end printing*/
         }
         else {
            *string_end = '\0';  /* start of "\330..." */
            fprintf (outfp, "(%s) show\n", string_start);
            /*
               gsave saves currentfont and currentpoint.
               grestore restores the default state: back to
               the listing font and currentpoint moves back to
               beginning of where glyph was printed.
            */
            fprintf (outfp, "gsave special (\\330) show\n");
            if (bold) {
               fprintf (outfp, "(\\330) stringwidth pop neg -0.5 rmoveto\n");
               fprintf (outfp, "(\\330) show 0 0.5 rmoveto\n");
            }
            fprintf (outfp, "grestore\n");
            /* move forward by width of one monospace glyph */
            fprintf (outfp, "(m) stringwidth rmoveto\n");
            /* for language-specific handling below */
            strncpy (string_end, "    ", 4);
            string_start = string_end + 4;  /* skip to next text to print */
         }
      }  /* while (outstring [current_start] != '\0') */
   }  /* (symbol == 1) */
   /*
      If Symbol font glyph is not present, print entire string.
   */
   else if (outstring[0] != '\0') {
      fprintf (outfp, "(%s) show\n", outstring);
   }

   return;
}


/*
   Draw tab stop lines on each page.
*/
void
new_ps_page_tabs (FILE *outfp, int *tab_stops, int wide) {

   int i;  /* loop variable */
   int tab_limit;  /* 80 if wide = 0, 132 otherwise */


   fprintf (outfp, "   %% Print dashed lines for every page\n");

   fprintf (outfp, "   gsave\n");
   fprintf (outfp, "   0.5 setgray [3] 0 setdash\n");

   tab_limit = (wide == 0) ? 81 : 133;

   for (i = 0;
        i <= tab_limit && tab_stops[i] > 0 && tab_stops[i] <= tab_limit;
        i++) {

      if (wide == 0) {
         fprintf (outfp, "   %d %d moveto 0 %d rlineto stroke\n",
                 LEFT_MARGIN_80 + 6 * (tab_stops[i] - 1),
                 TOP_MARGIN_80 + 20, -BOX_HEIGHT_80 - 22);
      }
      else {
         fprintf (outfp, "   %d %d moveto 0 %d rlineto stroke\n",
                 LEFT_MARGIN_132 + 6 * (tab_stops[i] - 1),
                 TOP_MARGIN_132 + 30, -BOX_HEIGHT_132 - 30);
      }
   }

   fprintf (outfp, "   grestore  %% Restore previous line type\n");
   fprintf (outfp, "   %% End of dashed lines\n");

   return;
}

